import boto3
from utils import constant

# conect to dynamodb
dynamodb = boto3.resource('dynamodb', region_name='us-east-1')
table = dynamodb.Table(constant.DYNAMODB_TABLE)

def get_organization_cursor():
    response = table.get_item(
        Key={
            'key': constant.DYNAMODB_ITEM
        }
    )

    res = []
    for org, cur in response['Item']['value'].items():
        res.append({"organization": org, "repository_cursor":cur})
    return res

def update_cursor(org:str, cur:str):

    table.update_item(
        Key={'key':constant.DYNAMODB_ITEM},
        ExpressionAttributeNames={
            "#v":"value",
            "#o":f"{org}"
        },
        ExpressionAttributeValues={
            ':v': f"{cur}"
        },
        UpdateExpression="SET #v.#o = :v",
    )

if __name__ == "__main__":
    print(get_organization_cursor())


import os

CONFIG_FILE = "configs/data_extraction.json"
API_URL = "https://api.github.com/graphql"
GITHUB_HEADER = {"Authorization": f"Bearer {os.getenv('github_token')}"}
BUCKET="github-datalake"
DYNAMODB_TABLE = "github"
DYNAMODB_ITEM = "AWS_stepfunction"
PARQUET_EXTENSION = ".parquet"
S3_FOLDER = "github"

ORGANIZATION_QUERY ="""
query {
  organization(login: "__organization_name__") {
   announcement
    createdAt
    description
    email
    name
    websiteUrl
    id
    location
    url
  }
}
"""

REPOSITORY_QUERY = """
query {
  organization(login: "__organization_name__") {
    repositories(first: 10, __cursor__) {
      pageInfo {
        endCursor
        hasNextPage
      }
      nodes {
        id
        createdAt
        name
        description
        url
      }
    }
  }
}
"""

LANGUAGE_QUERY = """
query {
    node(id: "__repository_id__") {
    ...on Repository{
      languages(first: 20, __cursor__){
        nodes{
          name
          id
          color
        }
        pageInfo {
          hasNextPage
          endCursor
        }
      }
    }
  }
}
"""

PULL_REQUEST_QUERY = """
query {
    node(id: "__repository_id__") {
    ...on Repository{
        pullRequests(first: 5, __cursor__){
          pageInfo{
            endCursor
            hasNextPage
          }
          nodes{
            body
            createdAt
            viewerSubscription
            id
          }
        }
      
    }
  }
}
"""

BRANCH_QUERY ="""
query {
  node(id: "__repository_id__") {
    ... on Repository {
      refs(refPrefix: "refs/heads/", first: 1, __branch_cursor__) {
        pageInfo {
          hasNextPage
          endCursor
        }
        nodes {
          name
          id
        }
      }
    }
  }
}
"""

COMMIT_QUERY = """
query {
  node(id: "__repository_id__") {
    ... on Repository {
      refs(refPrefix: "refs/heads/", first: 1, __branch_cursor__) {
        nodes {
          target {
            ... on Commit {
              history(first: 10, __commit_cursor__) {
                pageInfo {
                  hasNextPage
                  endCursor
                }
                nodes {
                  author {
                    name
                  }
                  message
                  committedDate
                  id
                }
              }
            }
          }
        }
      }
    }
  }
}
"""


import json
from utils import constant
import requests

url = constant.API_URL
headers = constant.GITHUB_HEADER

def handle_cursor(cursor:str):
    return "" if cursor.strip() == "" else "after: " + '"' + cursor + '"'
    
"""
['pageInfo']
    ['endCursor']
    ['hasNextPage']
['nodes']
    [
        ['id']
        ['createdAt']
        ['name']
        ['description']
        ['url']
    ]
"""
def extract_organization(organization_name:str):
    query = constant.ORGANIZATION_QUERY
    body = query.replace("__organization_name__", organization_name)
    try:
        response = requests.post(url, json={"query": body}, headers=headers)
        data = json.loads(response.content)
        return True, data['data']['organization']
    except Exception as e:
        print(e)
        return [False]

def extract_repository(organization_name:str, cursor = ""):
    query = constant.REPOSITORY_QUERY
    cursor = handle_cursor(cursor)
    body = query.replace("__organization_name__", organization_name)
    body = body.replace("__cursor__", cursor)
    try:
        response = requests.post(url, json={"query": body}, headers=headers)
        data = json.loads(response.content)
        return True, data['data']['organization']['repositories']
    except Exception as e:
        print(e)
        return [False]

def extract_pull_request(repository_id:str, cursor=""):
    query = constant.PULL_REQUEST_QUERY
    cursor = handle_cursor(cursor)
    body = query.replace("__repository_id__", repository_id)
    body = body.replace("__cursor__", cursor)
    try:
        response = requests.post(url, json={"query": body}, headers=headers)
        data = json.loads(response.content)
        return True, data['data']['node']['pullRequests']
    except Exception as e:
        print(e)
        return [False]
    
def extract_language(repository_id:str, cursor = ""):
    query = constant.LANGUAGE_QUERY
    cursor = handle_cursor(cursor)
    body = query.replace("__repository_id__", repository_id)
    body = query.replace("__cursor__", cursor)
    try:
        response = requests.post(url, json={"query": body}, headers=headers)
        data = json.loads(response.content)
        return True, data['data']['node']['languages']
    except Exception as e:
        print(e)
        return [False]

def extract_branch(repository_id:str, branch_cursor = ""):
    query = constant.BRANCH_QUERY
    branch_cursor = handle_cursor(branch_cursor)
    body = query.replace("__repository_id__", repository_id)
    body = query.replace("__branch_cursor__", branch_cursor)
    try:
        response = requests.post(url, json={"query": body}, headers=headers)
        data = json.loads(response.content)
        return True, data['data']['node']['refs']
    except Exception as e:
        print(e)
        return [False]

def extract_commit(repository_id:str, branch_cursor="", commit_cursor=""):
    query = constant.COMMIT_QUERY
    branch_cursor = handle_cursor(branch_cursor)
    commit_cursor = handle_cursor(commit_cursor)
    body = query.replace("__repository_id__", repository_id)
    body = body.replace("__branch_cursor__", branch_cursor)
    body = body.replace("__commit_cursor__", commit_cursor)
    try:
        response = requests.post(url, json={"query": body}, headers=headers)
        data = json.loads(response.content)
        return True, data['data']['node']['refs']['nodes'][-1]['target']['history']
    except Exception as e:
        print(e)
        return [False]



import boto3, json, io
import pandas as pd
from utils import constant

s3_client = boto3.client('s3')
s3_resource = boto3.resource('s3')

def save_json(id:str, content:dict):
    s3_client.put_object(Key=id, Bucket=constant.BUCKET, Body=json.dumps(content))

def read_json(file_key):
    response = s3_client.get_object(Bucket=constant.BUCKET, Key=file_key)
    file_content = response['Body'].read().decode('utf-8')
    json_content = json.loads(file_content)
    return json_content

def read_txt(file_key):
    print(read_txt)
    print(file_key)
    s3_object = s3_client.get_object(Bucket=constant.BUCKET, Key=file_key)
    body = s3_object['Body']
    return body.read().decode('utf-8')

def move_file(from_key, to_key):
    print('move file')
    print(from_key)
    print(to_key)
    copy_source = {
        'Bucket': constant.BUCKET,
        'Key': from_key
    }
    s3_resource.meta.client.copy(copy_source, constant.BUCKET, to_key)

def delete_file(folder_link):
    bucket = s3_resource.Bucket(constant.BUCKET)
    bucket.object_versions.filter(Prefix=folder_link).delete()

def save_parquet_to_s3(file_path:str, data:dict, parents:dict = {}):
    file_path = file_path + constant.PARQUET_EXTENSION
    df = pd.DataFrame.from_dict(data)

    for key, value in parents.items():
        df[key] = value

    out_buffer = io.BytesIO()
    df.to_parquet(out_buffer, index=False)

    s3_client.put_object(Bucket=constant.BUCKET, Key=file_path, Body=out_buffer.getvalue())



import setuptools 
  
with open("README.md", "r") as fh: 
    description = fh.read() 
  
setuptools.setup( 
    name="beerus-test-package", 
    version="0.0.2", 
    author="Beerus", 
    author_email="contact@gfg.com", 
    packages=["utils"], 
    description="A sample test package", 
    long_description=description, 
    long_description_content_type="text/markdown", 
    url="", 
    license='MIT', 
    python_requires='>=3.8', 
    install_requires=[] 
) 